# App Testing Environment - Reinforcement Learning Module

## Overview
This module is part of a PhD research project on "Automatisiertes Testen mit KI" and implements reinforcement learning based black-box testing for mobile applications. It works in conjunction with the white-box testing module to provide comprehensive testing capabilities through a containerized testing environment.

## Frontend Integration
This module can be accessed through the central testing platform frontend, which provides:
- Visual configuration of RL parameters
- Real-time monitoring of training progress
- Test execution status and results visualization
- Integration with the overall testing workflow

## Prerequisites:

To use this repository, you need the following installed:

- Docker (either via Windows Subsystem for Linux 2 (WSL2) or directly on a Linux OS). 
- Visual Studio Code with [Extensions](https://code.visualstudio.com/docs/devcontainers/create-dev-container).

Please note that the primary working directory is `/home/coder/workspace`.



## Getting Started

### WSL Install
The easierst way to install the repo with visual studio code and wsl is:
 - open wsl 
 - clone the repository in a directory of your choice
 - open VSC in the directory with `code .` 
 - VSC schould now ask if you want to open the workspace in docker.

Please note that the primary working directory is `/home/coder/workspace`.

### Clone the Repository

After opening the workspace in Coder, clone the repository into the workspace by running the following command:

```bash
git clone git@git.scc.kit.edu:von-bis/interne-software-projekte/apptestingenvironment.git
```

### Download Submodules

If you want to use a submodule, you can download it using the following commands:

```bash
git submodule update --init --recursive
git submodule update --remote --merge
```

### Create and Start an Android Emulator

1. To create an Android emulator, use the following command:

```bash
avdmanager create avd -n avd1 -k "system-images;android-33;google_apis;x86_64"
```

2. To start the emulator, use the following command:

```bash
emulator -avd "avd1" -noaudio -no-boot-anim -netdelay none -accel on -no-window -no-snapshot -memory 4096 -partition-size 4096 &
```

To see a window (if you use vnc or wsl)
```bash
emulator -avd "avd1" -noaudio -no-boot-anim -netdelay none -accel on -no-snapshot -memory 4096 -partition-size 4096 &
```

### Troubleshooting Emulator Startup

If you encounter a "kvm not found" error, run the following command on your machine:

```bash
chmod 777 /dev/kvm
```

If you continue to face problems, consider installing the following packages in your container: 

- libpulse0
- libxcomposite1
- libxcursor1

### Stopping the Emulator

To stop the emulator, use the following command:

```bash
adb -s emulator-5554 emu kill
```

The patch modifies the following:
- Deletes the adb path 
- Adds `-no-window` option

Please note that after every change to `/android_env` (including patches), you need to execute the previous steps. If you're using Jupyter notebooks, they will also need to be restarted after installing a new `android_env` version.

### Run Random Agent

To verify that everything was installed correctly, run the random agent from Deepmind. Note that the folder paths may need to be adjusted.

you can run the emulator headless (which works more often) by adding --run_headless=true

Here are some examples of how to run the random agent:
```python
python3 run_random_agent.py \
--avd_name='avd1' \
--android_avd_home=/home/coder/.android/avd/ \
--android_sdk_root=/opt/android-sdk \
--emulator_path=/opt/android-sdk/emulator/emulator \
--adb_path=/opt/android-sdk/platform-tools/adb \
--num_steps=1000 \
--task_path=/home/coder/workspace/rl_tasks/mdp_0007.textproto \
--run_headless='false'
```
If you don't have vnc or wsl you have to run it headless ('true' in the last line)
It might be, that the folder paths (here and in the textproto to the app) aren't correct. 

### Train in background

Usually if you close VSCode or delete the terminal the trainings ends. To prevent this you can start a tmux session with
```bash
tmux new -s session_name
```
Here you can start the run and detatch from the session by pressing 
```bash
ctrl + b then d
```
You can reattach to your session with
```bash
tmux attach -t session_name
```
In this way you can also train multiple agents at once, if you use different emulator instances.

### Run Training

To run the training, adjust the `train.sh` script according to your needs and then execute the following command:

```bash
./train.sh
```

## Repository Structure

Here's an overview of the repository's structure:

- [androidbuildtools](/androidbuildtools/): Contains necessary tools for installing a fully functional Android SDK environment.
- [requirements.txt](/requirements.txt): Lists all Python packages required for this repository.
- [androidtestapp](/androidtestapp/): Contains a simple Android app for testing purposes.
- [rl](/rl/): Contains the reinforcement learning code.
    - [networks](/rl/networks/): Contains the feature extractor network used to extract features from the environment.
    - [wrapper](/rl/wrapper/): Contains the wrappers used to modify the environment and learning process.
    - [main.py](/rl/main.py): Contains the main script for the reinforcement learning code.
    - [train.sh](/rl/train.sh): Contains the script for training the reinforcement learning code.
- [rl-tasks](/rl-tasks/): Contains tasks for the Android environment. These tasks are copied from the DeepMind repository.
- [sample_apps](/sampleApps/): Contains sample apps for testing.
