"""
Utility functions for working with APK files.
"""

import os
from androguard.core.bytecodes.apk import APK


def extract_info_from_apk(apk_path):
    """
    Extracts the package name and main activity from the given APK file.

    Parameters:
    - apk_path (str): The file path to the APK file.

    Returns:
    - package_name (str): The package name of the APK.
    - main_activity (str): The main activity of the APK.
    """

    # Load APK
    apk = APK(apk_path)

    # Retrieve the AndroidManifest.xml from the APK
    manifest = apk.get_android_manifest_xml()

    # Search for the main activity in the AndroidManifest.xml
    main_activity = None
    for activity in manifest.findall(".//activity"):
        if activity.find(".//intent-filter") is not None:
            main_activity = activity.get("{http://schemas.android.com/apk/res/android}name")
            break

    # Get the package name of the APK
    package_name = apk.package

    return package_name, main_activity


def is_apk(path):
    """
    Checks if the given file path is an APK file.

    Parameters:
    - path (str): The file path to check.

    Returns:
    - bool: True if the file path ends with '.apk', otherwise False.
    """
    return os.path.isfile and path.endswith(".apk")
