from typing import Any

from android_env.wrappers import base_wrapper
from dm_env import specs
import dm_env
import numpy as np

class NormalizeTouchActionWrapper(base_wrapper.BaseWrapper):
    """
    Normalize the touch action to a continuous float value between -1 and 1.
    This is not properly tested and might not work as expected.
    """
    def __init__(self, env: dm_env.Environment, config: dict):
        super().__init__(env)
        self.config = config
        
    def _process_action(self, action: dict[str, Any]) -> dict[str, Any]:
        if 'touch_position' in action:
            # Convert to NumPy array if it's not already
            if not isinstance(action['touch_position'], np.ndarray):
                action['touch_position'] = np.array(action['touch_position'], dtype=np.float32)

            old_action = action.copy()
            
            # Normalize to [-1, 1]
            action['touch_position'] = action['touch_position'] / 2 + 0.5
        else:
            raise ValueError("Action must contain 'touch_position' key")
        
        if not self.config["run_headless"]:
            print(f"[DEBUG] Old action: {old_action['touch_position']}, New action: {action['touch_position']}")
        
        return action

    def action_spec(self) -> dict[str, specs.Array]:
        spec = self._env.action_spec()
        spec['touch_position'] = specs.BoundedArray(
            shape=(2,), dtype=np.float32, minimum=-1.0, maximum=1.0)
        return spec